/*********************************************************
Copyright (c) CMX Systems, Inc. 2004. All rights reserved
*********************************************************/

#include "micronet.h"

SOCKET_INFO_T sock_info[NUM_SOCKETS];

static SCHAR next_socket(void) cmx_reentrant;

/********************************************************************/

/* Perform all non-RTOS initialization needed by MicroNet.
   Returns result from mn_ether_init if ethernet used, otherwise
   always returns 1. If an RTOS is used this must be called from a
   task and called after mn_os_init.
*/
int mn_init(void)
cmx_reentrant {
   int status;

   MEMSET(sock_info, 0, sizeof(sock_info));

   mn_ip_init();

   /* Note: mn_timer_init MUST be called before mn_uart_init and
      mn_ether_init.
   */
   mn_timer_init();

#if (IGMP)
   mn_igmp_init();
#endif      /* (IGMP) */

#if (BOOTP)
   mn_bootp_init();
#endif

#if DHCP
   mn_dhcp_init();
#endif      /*  DHCP */

#if VIRTUAL_FILE
   /* this assumes the virtual file system will be set up through the
      mn_??_set_entry functions.
   */
   MEMSET(vf_dir, 0, sizeof(vf_dir));
   MEMSET(page_send_info, 0, sizeof(page_send_info));
#if HTTP
#if (NUM_POST_FUNCS)
   MEMSET(pf, 0, sizeof(pf));
#endif      /* (NUM_POST_FUNCS) */
#if SERVER_SIDE_INCLUDES
   MEMSET(gf, 0, sizeof(gf));
#endif      /* #if SERVER_SIDE_INCLUDES */
#endif      /* #if HTTP */
#endif      /* VIRTUAL_FILE */

#if HTTP
   mn_http_init();
#endif      /*  HTTP */

#if (FTP && FTP_SERVER)
   mn_ftp_server_init();
#endif      /* #if (FTP && FTP_SERVER) */

#if SMTP
   mn_smtp_init();
#endif      /* #if SMTP */

#if ETHERNET
#if ARP
   mn_arp_init();
#endif      /* ARP */
   status = mn_ether_init();
#endif      /* ETHERNET */

#if (PPP && USE_PAP)
   MEMSET(pap_user, 0, sizeof(pap_user));
#endif      /* (PPP && USE_PAP) */

#if (PPP || SLIP)
   mn_uart_init();
   status = 1;
#endif      /* (PPP || SLIP) */

   if (status)
      {
      /* allow transmit - assumes semaphore was initialized to zero */
      MN_SIGNAL_POST(SIGNAL_TRANSMIT);
#if (POLLED_ETHERNET)
      /* Signal receive task it is ok to start. */
      MN_SIGNAL_POST(SIGNAL_RECEIVE);
#endif      /* (POLLED_ETHERNET) */
      }

   return (status);
}

/* fills socket structure and opens TCP connection if needed.
   returns socket_no or negative number on error.
*/
SCHAR mn_open(byte dest_ip[], word16 src_port, word16 dest_port, \
   byte open_mode, byte proto, byte type, byte *buff_ptr, word16 buff_len)
cmx_reentrant {
   SCHAR socket_no;
#if TCP
   PSOCKET_INFO socket_ptr;
#endif

#if (UDP && !TCP)
   open_mode = open_mode;
#endif

   socket_no = mn_open_socket(dest_ip, src_port, dest_port, proto,type, buff_ptr, buff_len);

#if TCP
      if (proto == PROTO_TCP)
         {
         if (socket_no >= 0)
            {
            socket_ptr = MK_SOCKET_PTR(socket_no);
            if (mn_tcp_open(open_mode,socket_ptr) <= 0)
               {
               (void)mn_abort(socket_no);
               return (TCP_OPEN_FAILED);
               }
            }
         }
#endif      /* TCP */

   return (socket_no);
}


/* fills socket structure. returns socket_no or negative number on error.
   mn_open is split into two functions because some linkers were complaining
   about recursion in tcp.c because mn_open was being called from
   tcp_recv_header. tcp_recv_header now calls mn_open_socket.
*/

SCHAR mn_open_socket(byte dest_ip[], word16 src_port, word16 dest_port, \
   byte proto, byte type, byte *buff_ptr, word16 buff_len)
cmx_reentrant {
   SCHAR socket_no;
   PSOCKET_INFO socket_ptr;

   /* Make sure we have a valid proto */
   socket_no = NOT_SUPPORTED;
   switch (proto)
      {
#if (ETHERNET)
      case PROTO_IP:
         if (type == S_ARP_TYPE)
            socket_no = 0;
         break;
#endif
#if (TCP)
      case PROTO_TCP:
#endif
#if (UDP)
      case PROTO_UDP:
#endif
#if (PING)
      case PROTO_ICMP:
#endif
#if (IGMP)
      case PROTO_IGMP:
#endif
         socket_no = 0;
         break;
      default:
         break;
      }

   if (socket_no == NOT_SUPPORTED)
      return (socket_no);

#if (ALLOW_MULTICAST)
   if (dest_ip[0] >= 224 && dest_ip[0] <= 239)
      type |= MULTICAST_TYPE;
#endif      /* (ALLOW_MULTICAST) */

   if ( src_port == 0 && (proto == PROTO_TCP || proto == PROTO_UDP) )
      socket_no = BAD_SOCKET_DATA;
#if (ALLOW_MULTICAST)
   else if (proto == PROTO_TCP && (type & MULTICAST_TYPE))
      socket_no = BAD_SOCKET_DATA;
   else if (dest_ip[0] == 224 && dest_ip[1] == 0 && dest_ip[2] == 0 \
         && dest_ip[3] == 0)
      /* Illegal multicast address */
      socket_no = BAD_SOCKET_DATA;
#else
   else if (type & MULTICAST_TYPE)
      socket_no = BAD_SOCKET_DATA;
#endif      /* (ALLOW_MULTICAST) */
   else
      {
      socket_ptr = mn_find_socket(src_port, dest_port, dest_ip, proto);
      if (socket_ptr != PTR_NULL && SOCKET_ACTIVE(socket_ptr->socket_no))
         socket_no = SOCKET_ALREADY_EXISTS;
      else
         {
         socket_no = next_socket();
         if (socket_no != NOT_ENOUGH_SOCKETS)
            {
            MN_TASK_LOCK;
            MEMSET(&sock_info[socket_no], 0, sizeof(SOCKET_INFO_T));
            socket_ptr = MK_SOCKET_PTR(socket_no);
            socket_ptr->socket_no = socket_no;
            socket_ptr->src_port = src_port;
            socket_ptr->dest_port = dest_port;
   
            socket_ptr->ip_dest_addr[0] = dest_ip[0];
            socket_ptr->ip_dest_addr[1] = dest_ip[1];
            socket_ptr->ip_dest_addr[2] = dest_ip[2];
            socket_ptr->ip_dest_addr[3] = dest_ip[3];
   
#if (ETHERNET)
#if (ARP)
            if (type == S_ARP_TYPE && proto == PROTO_IP)
#endif      /* ARP */
               {
               socket_ptr->eth_dest_hw_addr[0] = eth_dest_hw_addr[0];
               socket_ptr->eth_dest_hw_addr[1] = eth_dest_hw_addr[1];
               socket_ptr->eth_dest_hw_addr[2] = eth_dest_hw_addr[2];
               socket_ptr->eth_dest_hw_addr[3] = eth_dest_hw_addr[3];
               socket_ptr->eth_dest_hw_addr[4] = eth_dest_hw_addr[4];
               socket_ptr->eth_dest_hw_addr[5] = eth_dest_hw_addr[5];
               }
#endif   /* (ETHERNET) */
   
            socket_ptr->ip_proto = proto;
            socket_ptr->socket_type = type;
            if (buff_ptr && buff_len)
               {
               socket_ptr->recv_ptr = buff_ptr;
               socket_ptr->recv_end = buff_ptr+buff_len-1;
               }
   
            MN_SIGNAL_RESET(signal_socket[socket_no]);
   
#if TCP
            if (proto == PROTO_TCP)
               mn_tcp_init(socket_ptr);
#endif      /* TCP */
   
            SOCKET_ACTIVATE(socket_no);
            MN_TASK_UNLOCK;
            }
         }
      }

   return (socket_no);
}

/* sends a TCP or UDP packet using the passed socket. returns number of
   bytes sent or negative number on error. If a TCP packet waits for a reply.
*/
int mn_send(SCHAR socket_no, byte *data_ptr, word16 data_len)
cmx_reentrant {
   PSOCKET_INFO socket_ptr;
#if (DHCP)
   byte dhcp_state;
#endif      /* (DHCP) */
#if TCP
   int recvd;
   int retval;

   recvd = NEED_TO_LISTEN;
   retval = 0;
#endif

   if (socket_no < 0 || socket_no >= NUM_SOCKETS)
      return (BAD_SOCKET_DATA);

   if (!SOCKET_ACTIVE(socket_no))
      return (SOCKET_NOT_FOUND);

#if DHCP
   dhcp_state = dhcp_lease.dhcp_state;
   if (dhcp_state == DHCP_DEAD)
      return (DHCP_LEASE_EXPIRED);

#if (RTOS_USED == RTOS_NONE)
   if (bootpMode == BOOTP_INACTIVE)
      {
      if (dhcp_state == DHCP_RENEWING || dhcp_state == DHCP_REBINDING)
         {
         /* renew with the previous lease time */
         mn_dhcp_renew(dhcp_lease.org_lease_time);
         }
      }
#endif      /* (RTOS_USED == RTOS_NONE) */
#endif      /* DHCP */

#if (IGMP && RTOS_USED == RTOS_NONE)
   mn_igmp_check_timers();
#endif      /* (IGMP && RTOS_USED == RTOS_NONE) */

   socket_ptr = MK_SOCKET_PTR(socket_no);

   /* if no data_ptr passed, use the current value */
   if (data_ptr && data_len)
      {
      socket_ptr->send_ptr = data_ptr;
      socket_ptr->send_len = data_len;
      }

#if TCP
      if (socket_ptr->ip_proto == PROTO_TCP)
         {
         /* loop until we get an ack, a tcp_close or error */
         do
            {
            /* Send the Data */
            retval = mn_tcp_send(socket_ptr);
            /* Wait for a reply. */
            if (retval > 0)
               {
               while (recvd == NEED_TO_LISTEN)
                  recvd = mn_recv_wait(socket_no, 0, 0, (SOCKET_WAIT_TICKS));
               if (recvd < 0 && recvd != SOCKET_TIMED_OUT)  /* recv error */
                  {
                  retval = recvd;
                  break;
                  }
               }
            else        /* send error */
               break;
            }
         while (socket_ptr->tcp_unacked_bytes);

         return (retval);
         }

#endif      /* TCP */

#if UDP
      if (socket_ptr->ip_proto == PROTO_UDP)
         return (mn_udp_send(socket_ptr));
#endif

      return (0);
}

/* loops until a TCP or UDP packet is received for the passed socket.
   returns number of bytes received or negative number on error.
   Also returns if no packet received in SOCKET_WAIT_TICKS timer ticks.
*/
int mn_recv(SCHAR socket_no, byte *buff_ptr, word16 buff_len)
cmx_reentrant {
   /* this function is set up like this to avoid breaking existing code */
   return (mn_recv_wait(socket_no, buff_ptr, buff_len, (SOCKET_WAIT_TICKS)));
}

/* loops until a TCP or UDP packet is received for the passed socket.
   returns number of bytes received or negative number on error.
   Also returns if no packet received in wait_ticks timer ticks.
   If wait_ticks is 0 will return immediately if no packet available.
*/
int mn_recv_wait(SCHAR socket_no, byte *buff_ptr, word16 buff_len, word16 wait_ticks)
cmx_reentrant {
   int retval;
   PSOCKET_INFO socket_ptr;
#if (RTOS_USED == RTOS_NONE)
   PSOCKET_INFO org_socket_ptr;
   TIMER_INFO_T sock_timer;
   byte packet_type;
#endif      /* (RTOS_USED == RTOS_NONE) */

   retval = 0;

   if (socket_no < 0 || socket_no >= NUM_SOCKETS)
      return (BAD_SOCKET_DATA);

   if (!SOCKET_ACTIVE(socket_no))
      return (SOCKET_NOT_FOUND);

#if DHCP
   if (dhcp_lease.dhcp_state == DHCP_DEAD)
      return (DHCP_LEASE_EXPIRED);
#endif      /* DHCP */

   socket_ptr = MK_SOCKET_PTR(socket_no);

   /* UDP sockets need a receive buffer, but with TCP we can handle some
      types of packets without a receive buffer for the socket.
   */
   if (buff_ptr && buff_len)
      {
      socket_ptr->recv_ptr = buff_ptr;
      socket_ptr->recv_end = buff_ptr+buff_len-1;
      }

#if UDP
   if (socket_ptr->ip_proto == PROTO_UDP && socket_ptr->recv_ptr == PTR_NULL)
      return (BAD_SOCKET_DATA);
#endif

#if (RTOS_USED == RTOS_NONE)
   org_socket_ptr = socket_ptr;

   if (wait_ticks)
      mn_reset_timer(&sock_timer, wait_ticks);

   while (1)
      {
      /* loop until we get a tcp or udp packet or timeout */
      packet_type = mn_ip_recv();
      if (!(packet_type & (TCP_TYPE|UDP_TYPE|ICMP_TYPE)))
         {
         /* if we get here either there was no packet or we received a
            packet of a bad or unknown type.
         */
         if (!wait_ticks || mn_timer_expired(&sock_timer))
            return (SOCKET_TIMED_OUT);
         else
            {
#if (IGMP && RTOS_USED == RTOS_NONE)
            mn_igmp_check_timers();
#endif      /* (IGMP && RTOS_USED == RTOS_NONE) */

            /* call the callback function so something can be done while
               we are waiting.
            */
            if (mn_app_recv_idle() == NEED_TO_EXIT)
               return (NEED_TO_EXIT);
            else
               continue;
            }
         }

      if (packet_type & ICMP_TYPE)
         continue;

#if TCP
      if (packet_type & TCP_TYPE)
         {
         retval = mn_tcp_recv(&socket_ptr);
         if (retval == TCP_NO_CONNECT)
            break;
         }
#endif
#if TCP && UDP
      else
#endif
#if UDP
         {
         if (packet_type & UDP_TYPE)
            {
            retval = mn_udp_recv(&socket_ptr);
            }
         }
#endif
      /* if we got a packet for a different socket, get another packet.
      */
      if (socket_ptr != org_socket_ptr)
         socket_ptr = org_socket_ptr;
      else
         break;
      }

#else    /* using an RTOS */
   if (!wait_ticks)
      {
      /* don't wait for a signal, just check for one */
      if (MN_SIGNAL_GET(signal_socket[socket_ptr->socket_no]) == SIGNAL_SUCCESS)
         {
         MN_TASK_LOCK;
         retval = socket_ptr->last_return_value;
         MN_TASK_UNLOCK;
         }
      else
         retval = SOCKET_TIMED_OUT;
      }
   else
      {
      if (MN_SIGNAL_WAIT(signal_socket[socket_ptr->socket_no],wait_ticks) == \
            SIGNAL_SUCCESS)
         {
         MN_TASK_LOCK;
         retval = socket_ptr->last_return_value;
         MN_TASK_UNLOCK;
         }
      else
         retval = SOCKET_TIMED_OUT;
      }
#endif      /* (RTOS_USED == RTOS_NONE) */

   return (retval);
}

/* does a tcp_shutdown if a tcp socket and closes the socket. returns
   negative number if a bad socket passed, otherwise returns 0.
*/
int mn_close(SCHAR socket_no)
cmx_reentrant {
   PSOCKET_INFO socket_ptr;

   if (socket_no < 0 || socket_no >= NUM_SOCKETS)
      return (BAD_SOCKET_DATA);

   if (!SOCKET_ACTIVE(socket_no))
      return (SOCKET_NOT_FOUND);

#if DHCP
   if (dhcp_lease.dhcp_state == DHCP_DEAD)
      {
      mn_abort(socket_no);
      return (DHCP_LEASE_EXPIRED);
      }
#endif      /* DHCP */

   socket_ptr = MK_SOCKET_PTR(socket_no);

#if TCP
   if (socket_ptr->ip_proto == PROTO_TCP)
      {
      mn_tcp_shutdown(socket_ptr);
      }
#endif

   MN_TASK_LOCK;
   CLEAR_SOCKET(socket_no);
   socket_ptr->src_port = 0;
   MN_SIGNAL_RESET(signal_socket[socket_no]);
   MN_TASK_UNLOCK;
   return (0);
}

/* does a tcp_abort if a tcp socket and closes the socket. returns
   negative number if a bad socket passed, otherwise returns 0. The
   difference between mn_close and mn_abort is that mn_abort will not
   send any packets, while mn_close for tcp sockets will send a TCP FIN
   and negotiate a close.
*/
int mn_abort(SCHAR socket_no)
cmx_reentrant {
   PSOCKET_INFO socket_ptr;

   if (socket_no < 0 || socket_no >= NUM_SOCKETS)
      return (BAD_SOCKET_DATA);

   if (!SOCKET_ACTIVE(socket_no))
      return (SOCKET_NOT_FOUND);

   socket_ptr = MK_SOCKET_PTR(socket_no);

#if TCP
   if (socket_ptr->ip_proto == PROTO_TCP)
      {
      mn_tcp_abort(socket_ptr);
      }
#endif

   MN_TASK_LOCK;
   CLEAR_SOCKET(socket_no);
   socket_ptr->src_port = 0;
   MN_SIGNAL_RESET(signal_socket[socket_no]);
   MN_TASK_UNLOCK;
   return (0);
}

/* finish the packet and turn on xmit ISR, len is only used by ethernet.
   returns 1 if successful or less than 0 if an error.

   PPP and SLIP always return 1.
*/
SCHAR mn_close_packet(PSOCKET_INFO socket_ptr, word16 len)
cmx_reentrant {
#if (ETHERNET)
   int status;
#endif      /* ETHERNET */

#if DHCP
   if (dhcp_lease.dhcp_state != DHCP_DEAD)
      {
#endif      /* DHCP */

#if TCP
      if (socket_ptr != PTR_NULL && socket_ptr->ip_proto == PROTO_TCP)
         socket_ptr->tcp_resends -= 1;
      else
#endif
         socket_ptr = socket_ptr;

#if (ETHERNET)
   /* the CS8900 automatically generates the CRC, if your ethernet chip
      doesn't, add code to do it here.
   */
      MN_XMIT_BUSY_SET;
      status = ETHER_SEND(socket_ptr, len);
#if (POLLED_ETHERNET)
      MN_XMIT_BUSY_CLEAR;
      MN_SIGNAL_POST(SIGNAL_TRANSMIT);    /* allow transmit */
#else
      if (status <= 0)
         {
         MN_XMIT_BUSY_CLEAR;
         MN_SIGNAL_POST(SIGNAL_TRANSMIT);    /* allow transmit */
         }
#endif      /* (POLLED_ETHERNET) */

/*      return (SCHAR)(status > 0 ? 1 : (SCHAR)status); */
      if (status > 0)
         return (1);
      else
         return ((SCHAR)status);

#elif PPP
      len=len;                   /* keep compiler happy */
      mn_ppp_close_packet();
      return (1);
#elif SLIP
      len=len;                   /* keep compiler happy */
      mn_send_byte(SLIP_END);
      XMIT_INT_ON;
      return (1);
#endif

#if DHCP
      }
   else
      return (DHCP_LEASE_EXPIRED);
#endif      /* DHCP */

}

/* Determines socket_type from the src_port. The socket_type is used in
   the callback functions to do special processing. (See callback.c).
*/
byte mn_get_socket_type(word16 src_port)
cmx_reentrant {
   byte retval;

   switch (src_port)
      {
      case HTTP_PORT:
         /* HTTP sockets always use inactivity timers */
         retval = (HTTP_TYPE | INACTIVITY_TIMER);
         break;
      case FTP_DATA_PORT:
      case FTP_CONTROL_PORT:
         retval = FTP_TYPE;
         break;
      case SMTP_PORT:
         retval = SMTP_TYPE;
         break;
      case TFTP_SERVER_PORT:
         retval = TFTP_TYPE;
         break;
      default:
         retval = STD_TYPE;
         break;
      }

   return (retval);
}

/* return a pointer to a socket matching the passed ports, ip_dest_addr and
   prototype.
*/
PSOCKET_INFO mn_find_socket(word16 src_port, word16 dest_port, \
   byte *dstaddr, byte proto)
cmx_reentrant {
   byte i;
   PSOCKET_INFO socket_ptr;
#if (BOOTP || DHCP)
   SCHAR bp_mode;

   bp_mode = bootpMode;
#endif      /* (BOOTP || DHCP) */

   socket_ptr = (PSOCKET_INFO)PTR_NULL;

   if (dstaddr != PTR_NULL)
      {
      for (i=0;i<NUM_SOCKETS;i++)
         {
         if (sock_info[i].src_port == src_port && \
               sock_info[i].dest_port == dest_port && \
               sock_info[i].ip_proto == proto)
            {
#if (BOOTP || DHCP)
            if (( (bp_mode == BOOTP_ACTIVE) && \
                  memcmp((void *)sock_info[i].ip_dest_addr, (void *)broadcast_addr, \
                     IP_ADDR_LEN) == 0) || \
                  memcmp((void *)sock_info[i].ip_dest_addr, (void *)dstaddr, \
                     IP_ADDR_LEN) == 0)
#else
            if (memcmp((void *)sock_info[i].ip_dest_addr, (void *)dstaddr, \
                  IP_ADDR_LEN) == 0)
#endif      /* (BOOTP || DHCP) */
               {
               socket_ptr = &sock_info[i];
               break;
               }
            }
         }
      }

   return (socket_ptr);
}

/* searches for an available socket, returns socket number or negative #
   if none found.
*/
static SCHAR next_socket(void)
cmx_reentrant {
   SCHAR i;
   SCHAR socket_no;

   socket_no = NOT_ENOUGH_SOCKETS;

   for (i=0;i<NUM_SOCKETS;i++)
      {
      if (!SOCKET_ACTIVE(i))
         {
         socket_no = i;
         break;
         }
      }
   return (socket_no);
}

/* SR */
extern int cs8900_get_link_status(void) cmx_reentrant;

int mn_get_link_status(void) cmx_reentrant
{
   return cs8900_get_link_status();
}

#if (SOCKET_INACTIVITY_TIME)
#if defined(__C51__)    /* Keil 8051 */
#pragma NOAREGS
#endif      /* defined(__C51__) */
#if (defined(POLECOG1) || defined(POLECOG1E))
void __irq_code mn_update_inactivity_timers(void)
#elif (defined(_CC51))     /* Tasking 8051 */
_noregaddr void mn_update_inactivity_timers(void)
#else
void mn_update_inactivity_timers(void)
#endif
cmx_reentrant {
   static word16 count = 0;      /* used to convert timer-ticks to seconds */
   PSOCKET_INFO socket_ptr;
   SCHAR i;

   count++;

   if (count == (ONE_SECOND))
      {
      count = 0;

      for (i=0;i<NUM_SOCKETS;i++)
         {
         if (SOCKET_ACTIVE(i))
            {
            socket_ptr = MK_SOCKET_PTR(i);
            if (socket_ptr->socket_type & INACTIVITY_TIMER)
               {
               if (socket_ptr->inactivity_timer)
                  {
                  socket_ptr->inactivity_timer--;
                  if (!(socket_ptr->inactivity_timer))
                     {
#if (defined(POLECOG1) || defined(CMXECOG1) || defined(POLECOG1E) || defined(CMXECOG1E))
#if TCP
                     if (socket_ptr->ip_proto == PROTO_TCP)
                        {
                        socket_ptr->tcp_state = TCP_CLOSED;
                        socket_ptr->send_ptr = BYTE_PTR_NULL;
                        socket_ptr->send_len = 0;
                        socket_ptr->tcp_unacked_bytes = 0;
                        }
#endif
                     CLEAR_SOCKET(i);
                     socket_ptr->src_port = 0;
#else
                     mn_abort(i);
#endif
                     }
                  }
               }
            }
         }
      }
}
#if defined(__C51__)    /* Keil 8051 */
#pragma AREGS
#endif      /* defined(__C51__) */

void mn_reset_inactivity_timer(PSOCKET_INFO socket_ptr)
cmx_reentrant {
   if (socket_ptr->socket_type & INACTIVITY_TIMER)
      {
      DISABLE_INTERRUPTS;
      /* If the following is an atomic instruction the calls to
         DISABLE_INTERRUPTS and ENABLE_INTERRUPTS may be removed.
      */
      socket_ptr->inactivity_timer = SOCKET_INACTIVITY_TIME;
      ENABLE_INTERRUPTS;
      }
}

#endif      /* (SOCKET_INACTIVITY_TIME) */
